@@ -1,3 +1,33 @@
+# Copyright or © or Copr. Loïc Paulevé (2023)
+#
+# loic.pauleve@cnrs.fr
+#
+# This software is governed by the CeCILL license under French law and
+# abiding by the rules of distribution of free software.  You can  use,
+# modify and/ or redistribute the software under the terms of the CeCILL
+# license as circulated by CEA, CNRS and INRIA at the following URL
+# "http://www.cecill.info".
+#
+# As a counterpart to the access to the source code and  rights to copy,
+# modify and redistribute granted by the license, users are provided only
+# with a limited warranty  and the software's author,  the holder of the
+# economic rights,  and the successive licensors  have only  limited
+# liability.
+#
+# In this respect, the user's attention is drawn to the risks associated
+# with loading,  using,  modifying and/or developing or reproducing the
+# software by the user in light of its specific status of free software,
+# that may mean  that it is complicated to manipulate,  and  that  also
+# therefore means  that it is reserved for developers  and  experienced
+# professionals having in-depth computer knowledge. Users are therefore
+# encouraged to load and test the software's suitability as regards their
+# requirements in conditions enabling the security of their systems and/or
+# data to be ensured and,  more generally, to use and operate it in the
+# same conditions as regards security.
+#
+# The fact that you are presently reading this means that you have had
+# knowledge of the CeCILL license and that you accept its terms.
+#
 
 import itertools
 import multiprocessing
@@ -101,16 +131,24 @@ class BonesisView(object):
     def __next__(self):
         if self.limit and self._counter >= self.limit:
             raise StopIteration
+
         t = Timer(self.settings["timeout"], self.interrupt) \
                 if "timeout" in self.settings else None
         t.start() if t is not None else None
         try:
             self.cur_model = next(self._iterator)
+            if self.mode == "opt":
+                try:
+                    while True:
+                        self.cur_model = next(self._iterator)
+                except StopIteration:
+                    pass
+            elif self.mode == "optN":
+                while not self.cur_model.optimality_proven:
+                    self.cur_model = next(self._iterator)
         finally:
             t.cancel() if t is not None else None
-        if self.mode.startswith("opt") \
-                and not self.cur_model.optimality_proven:
-            return next(self)
+
         pmodel = self.parse_model(self.cur_model)
         for func in self.filters:
             if not func(pmodel):
@@ -302,29 +340,31 @@ class DiverseBooleanNetworksView(BooleanNetworksView):
 
 class ConfigurationView(BonesisView):
     project = True
-    def __init__(self, cfg, *args, **kwargs):
+    _pred_name = "cfg"
+    def __init__(self, cfg, *args, scope=None, **kwargs):
         super().__init__(*args, **kwargs)
         self.cfg = cfg
+        self.scope = scope
     def configure_show(self):
         name = symbol_of_py(self.cfg.name)
-        self.control.add("base", [],
-            "#show."
-            f"#show cfg(X,N,V) : cfg(X,N,V), X={name}.")
+        self.control.add("base", [], "#show.")
+        if self.scope is not None:
+            for n in self.scope:
+                n = symbol_of_py(n)
+                self.control.add("base", [], f"show_scope({self._pred_name}({name},{n})).")
+            self.control.add("base", [], f"#show {self._pred_name}(X,N,V) : "
+                             f"{self._pred_name}(X,N,V), X={name},"
+                             f"show_scope({self._pred_name}(X,N)).")
+        else:
+            self.control.add("base", [], f"#show {self._pred_name}(X,N,V) :"
+                            f"{self._pred_name}(X,N,V), X={name}.")
     def format_model(self, model):
         atoms = model.symbols(shown=True)
         x = self.cfg.name
         return configurations_of_facts(atoms, keys=[x])[x]
 
-class HypercubeView(BonesisView):
-    project = True
-    def __init__(self, h, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.h = h
-    def configure_show(self):
-        name = symbol_of_py(self.h.name)
-        self.control.add("base", [],
-            "#show."
-            f"#show hypercube(X,N,V) : hypercube(X,N,V), X={name}.")
+class HypercubeView(ConfigurationView):
+    _pred_name = "hypercube"
     def format_model(self, model):
         pairs = []
         for a in model.symbols(shown=True):
@@ -378,7 +418,7 @@ def SomeFreezeComplementaryView(some, *args, **kwargs):
 
         nodes = list(some.mgr.bo.domain)
         if opts["exclude"]:
-            nodes = filter(lambda n: n not in opts["exclude"], nodes)
+            nodes = [n for n in nodes if n not in opts["exclude"]]
         elements = [(n,0) for n in nodes] + [(n,1) for n in nodes]
 
         def freeze_add(fs, e):
